<?php

if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * Generate the Form Design Selector to PMS -> General Settings
 *
 */
function pms_render_forms_design_selector() {

    $form_designs_data = pms_get_form_designs_data();

    wp_enqueue_script( 'jquery-ui-dialog' );

    $output = '<div id="pms-forms-design-browser">';

    foreach ( $form_designs_data as $form_design ) {

        if ($form_design['status'] == 'active') {
            $status = ' active';
            $title = '<strong>Active: </strong> ' . $form_design['name'];
        } else {
            $status = '';
            $title = $form_design['name'];
        }

        if ( $form_design['id'] != 'form-style-default' )
            $preview_button = '<div class="pms-forms-design-preview button-secondary" id="'. $form_design['id'] .'-info">Preview</div>';
        else $preview_button = '';

        $output .= '
                <div class="pms-forms-design'. $status .'" id="'. $form_design['id'] .'">
                    <label for="pms-fd-option-' . $form_design['id'] . '">
                        <input type="radio" id="pms-fd-option-' . $form_design['id'] . '" value="' . $form_design['id'] . '" ' . (( $form_design['status'] === 'active' ) ? 'checked="checked"' : '') . ' name="pms_general_settings[forms_design]">
                        ' . $form_design['name'] . '

                       <div class="pms-forms-design-screenshot">
                          <img src="' . $form_design['images']['main'] . '" alt="Form Design">
                          '. $preview_button .'
                       </div>
                   </label>
                </div>
        ';

        $img_count = 0;
        $image_list = '';
        foreach ( $form_design['images'] as $image ) {
            $img_count++;
            $active_img = ( $img_count == 1 ) ? ' active' : '';
            $image_list .= '<img class="pms-forms-design-preview-image'. $active_img .'" src="'. $image .'">';
        }

        if ( $img_count > 1 ) {
            $previous_button = '<div class="pms-slideshow-button pms-forms-design-sildeshow-previous disabled" data-theme-id="'. $form_design['id'] .'" data-slideshow-direction="previous"> < </div>';
            $next_button = '<div class="pms-slideshow-button pms-forms-design-sildeshow-next" data-theme-id="'. $form_design['id'] .'" data-slideshow-direction="next"> > </div>';
            $justify_content = 'space-between';
        }
        else {
            $previous_button = $next_button = '';
            $justify_content = 'center';
        }

        $output .= '<div id="pms-modal-'. $form_design['id'] .'" class="pms-forms-design-modal" title="'. $form_design['name'] .'">
                        <div class="pms-forms-design-modal-slideshow" style="justify-content: '. $justify_content .'">
                            '. $previous_button .'
                            <div class="pms-forms-design-modal-images">
                                '. $image_list .'
                            </div>
                            '. $next_button .'
                        </div>
                    </div>';

    }

    $output .= '</div>';

    if ( defined( 'WPPB_PAID_PLUGIN_DIR' ) && file_exists( WPPB_PAID_PLUGIN_DIR.'/features/form-designs/form-designs.php' ) ){
        $pb_generalSettings = get_option( 'wppb_general_settings' );
        $pb_styles = array(
            'form-style-default' => 'Default Style',
            'form-style-1' => 'Sublime',
            'form-style-2' => 'Greenery',
            'form-style-3' => 'Slim'
        );

        if ( !isset( $pb_generalSettings['formsDesign'] ) )
            $pb_active_form_style = 'Default Style';
        else $pb_active_form_style = $pb_styles[$pb_generalSettings['formsDesign']];

        $output .= '<p class="cozmoslabs-description">'. sprintf( esc_html__( 'For a consistent design on your website, it is best to set the same Form Style for both %1$sPaid Member Subscriptions%2$s and %1$sProfile Builder%2$s plugins.', 'paid-member-subscriptions' ),'<strong>', '</strong>' ) .'</p>';
        $output .= '<p class="cozmoslabs-description">'. sprintf( esc_html__( 'The currently active Form Style for Profile Builder forms is:  %1$s %3$s %2$s.', 'paid-member-subscriptions' ),'<strong>', '</strong>', $pb_active_form_style ) .'</p>';
    }

    return $output;
}


/**
 * Get Form Designs Data
 *
 */
function pms_get_form_designs_data() {
    $active_design = pms_get_active_form_design();

    $form_designs = array(
        array(
            'id' => 'form-style-default',
            'name' => 'Default Style',
            'status' => $active_design == 'form-style-default' ? 'active' : '',
            'images' => array(
                'main' => PMS_PLUGIN_DIR_URL.'assets/images/pms-fd-style-default.jpg',
            ),
        ),
        array(
            'id' => 'form-style-1',
            'name' => 'Sublime',
            'status' => $active_design == 'form-style-1' ? 'active' : '',
            'images' => array(
                'main' => PMS_PLUGIN_DIR_URL.'assets/images/pms-fd-style1-slide1.jpg',
                'slide1' => PMS_PLUGIN_DIR_URL.'assets/images/pms-fd-style1-slide2.jpg',
            ),
        ),
        array(
            'id' => 'form-style-2',
            'name' => 'Greenery',
            'status' => $active_design == 'form-style-2' ? 'active' : '',
            'images' => array(
                'main' => PMS_PLUGIN_DIR_URL.'assets/images/pms-fd-style2-slide1.jpg',
                'slide1' => PMS_PLUGIN_DIR_URL.'assets/images/pms-fd-style2-slide2.jpg',
            ),
        ),
        array(
            'id' => 'form-style-3',
            'name' => 'Slim',
            'status' => $active_design == 'form-style-3' ? 'active' : '',
            'images' => array(
                'main' => PMS_PLUGIN_DIR_URL.'assets/images/pms-fd-style3-slide1.jpg',
                'slide1' => PMS_PLUGIN_DIR_URL.'assets/images/pms-fd-style3-slide2.jpg',
            ),
        )
    );

    return $form_designs;
}


/**
 * Get the Form Designs active Style
 *
 */
function pms_get_active_form_design() {
    $wppb_generalSettings = get_option( 'pms_general_settings' );

    if ( empty( $wppb_generalSettings['forms_design'] ) || $wppb_generalSettings['forms_design'] == 'form_style_default')
        $active_design = 'form-style-default';
    else $active_design = $wppb_generalSettings['forms_design'];

    return $active_design;
}

/**
 * Add Form Design active Style wrapper
 */
function pms_add_form_styling( $output, $extra_data ) {
    $active_design = pms_get_active_form_design();

    if ( $active_design == 'form-style-default'  )
        return $output;

    $success_notice_title = '<span class="pms-notice-title">' . __('SUCCESS!', 'paid-member-subscriptions') . '</span>';
    if ( !strpos( $output, $success_notice_title ) && preg_match( '/<div class="pms_success-messages-wrapper">[\r\n\s]*<p>/', $output ) ) {
        $output = preg_replace( '/(<div class="pms_success-messages-wrapper">[\r\n\s]*<p>)/', '$1' . $success_notice_title, $output );
    }

    //$account_details_section_title = '<h3 class="pms-account-details-title">'. __('Account Details', 'paid-member-subscriptions') .'</h3>';
    //$subscription_plans_section_title = '<h3 class="pms-subscriptions-list-title">'. __('Select Your Subscription Plan', 'paid-member-subscriptions') .'</h3>';
    $group_plan_details_section_title = '<h4 class="pms-group-details-title">'. __('Add Your Group Details', 'paid-member-subscriptions') .'</h4>';

    // if ( strpos( $output, '<ul class="pms-form-fields-wrapper' ) !== false && ( !is_user_logged_in() || $extra_data == 'profile' ) && $extra_data != 'recover_password_form' )
    //     $output = str_replace( '<ul class="pms-form-fields-wrapper', $account_details_section_title . '<ul class="pms-form-fields-wrapper', $output );

    // if ( strpos( $output, '<li class="pms-field pms-field-subscriptions' ) !== false && strpos( $output, '<div class="pms-subscription-plan ' ) !== false && !isset( $_GET['subscription_id'] ) )
    //     $output = str_replace( '<li class="pms-field pms-field-subscriptions', $subscription_plans_section_title . '<li class="pms-field pms-field-subscriptions', $output );

    if ( strpos( $output, '<div class="pms-field pms-group-name-field pms-group-memberships-field' ) !== false && strpos( $output, $group_plan_details_section_title) === false )
        $output = str_replace( '<div class="pms-field pms-group-name-field pms-group-memberships-field', $group_plan_details_section_title . '<div class="pms-field pms-group-name-field pms-group-memberships-field', $output );

    // remove any existing Form Design wrapper before adding a new one
    if ( strpos( $output, '<div class="pms-form-design-wrapper" id="pms-'. $active_design .'-wrapper">' ) !== false )
        $output = pms_remove_form_design_wrapper( $output, 'pms-'. $active_design .'-wrapper' );

    $edited_output = '<div class="pms-form-design-wrapper" id="pms-'. $active_design .'-wrapper">'; // the wrapper helps when overwriting form styles (more specific targeting)
    $edited_output .= $output;
    $edited_output .= '</div>';

    return $edited_output;
}
add_filter( 'pms_register_shortcode_content', 'pms_add_form_styling', 10, 2 );
add_filter( 'pms_account_shortcode_content', 'pms_add_form_styling', 10, 2 );
add_filter( 'pms_login_form_shortcode_content', 'pms_add_form_styling', 10, 2 );
add_filter( 'pms_recover_password_shortcode_content', 'pms_add_form_styling', 10, 2 );
add_filter( 'pms_change_subscription_shortcode_content', 'pms_add_form_styling', 10, 2 );
add_filter( 'pms_retry_payment_shortcode_content', 'pms_add_form_styling', 10, 2 );
add_filter( 'pms_group_membership_dashboard_content', 'pms_add_form_styling', 10, 2 );
add_filter( 'pms_cancel_subscription_form_content', 'pms_add_form_styling', 10, 2 );
add_filter( 'pms_abandon_subscription_form_content', 'pms_add_form_styling', 10, 2 );
add_filter( 'pms_pb_email_confirmation_payment_form_content', 'pms_add_form_styling', 10, 2 );
add_filter( 'pms_update_payment_method_form_content', 'pms_add_form_styling', 10, 2 );
add_filter( 'pms_paypal_express_confirmation_form_content', 'pms_add_form_styling', 10, 2 );
add_filter( 'pms_payment_history_shortcode_content', 'pms_add_form_styling', 10, 2 );
add_filter( 'pms_subscriptions_form_content', 'pms_add_form_styling', 10, 2 );


/**
 * Filter the Renew Subscription Plan Form
 */
function pms_add_renew_form_styling( $output, $content, $user_id, $subscription_plan ) {
    return pms_add_form_styling( $output, 'renew_subscription_form' );
}
add_filter( 'pms_the_content_member_renew_subscription', 'pms_add_renew_form_styling', 10, 4 );


/**
 * Filter the Recover Password Form
 */
function pms_recover_password_message_styling( $output ) {
    return pms_add_form_styling( $output, 'recover_password_message' );
}
add_filter( 'pms_recover_password_confirmation_link_message', 'pms_recover_password_message_styling' );


/**
 * Filter the Login Form
 */
function pms_login_form_logged_in_message_styling( $output ) {
    return pms_add_form_styling( $output, 'login_form_logged_in_message' );
}
add_filter( 'pms_login_form_logged_in_message', 'pms_login_form_logged_in_message_styling' );


/**
 * Remove existing Form Design wrapper
 */
function pms_remove_form_design_wrapper( $output, $wrapper_id ) {

    // don't display potential HTML structure errors or warnings
    libxml_use_internal_errors(true);

    // load the HTML without automatically inserting the DOCTYPE declaration, or the <html> and <body> tags
    $html = new DOMDocument;
    $html->loadHTML( $output, LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD );

    $wrapper = $html->getElementById( $wrapper_id );

    if ( $wrapper ) {
        $wrapper_parent = $wrapper->parentNode;

        // move the wrapper’s content outside
        while ( $wrapper->firstChild ) {
            $wrapper_parent->insertBefore( $wrapper->firstChild, $wrapper );
        }

        $wrapper_parent->removeChild( $wrapper );
    }

    libxml_clear_errors();

    return $html->saveHTML();
}


/**
 * Load Form Design Feature Scripts and Styles
 *
 */
function pms_enqueue_form_design_styles() {
    $active_design = pms_get_active_form_design();

    if ( $active_design == 'form-style-default' )
        return;

    $file_path = plugin_dir_url( __FILE__ ) . 'css/pms-fd-'. $active_design .'.css';

    wp_register_style( 'pms_form_designs_style', $file_path, array(),PMS_VERSION );
    wp_enqueue_style( 'pms_form_designs_style' );

    wp_enqueue_style( 'pms-style-front-end', PMS_PLUGIN_DIR_URL . 'assets/css/style-front-end.css', array(), PMS_VERSION );
}
add_action('wp_enqueue_scripts' , 'pms_enqueue_form_design_styles');
add_action('elementor/editor/after_enqueue_styles' , 'pms_enqueue_form_design_styles');
add_action( 'enqueue_block_editor_assets', 'pms_enqueue_form_design_styles' );

function pms_enqueue_form_design_scripts() {
    $active_design = pms_get_active_form_design();

    if ( $active_design == 'form-style-default' )
        return;

    $file = 'pms-fd-front-end.js';

    $file_path = plugin_dir_url( __FILE__ ) . 'js/'.$file;

    wp_enqueue_script( 'pms_form_designs_script', $file_path, array( 'jquery' ), PMS_VERSION );

}
add_action('wp_enqueue_scripts' , 'pms_enqueue_form_design_scripts');
add_action('elementor/editor/after_enqueue_scripts' , 'pms_enqueue_form_design_scripts');


/**
 * Remove PB Form Designs Feature wrapper class and extra title
 *
 */
function pms_clear_ec_payment_form_pb_styling( $form_content ) {

    $pb_fd_wrappers = array('wppb-form-style-1-wrapper', 'wppb-form-style-2-wrapper', 'wppb-form-style-3-wrapper');

    foreach ( $pb_fd_wrappers as $wrapper ) {
        if ( strpos( $form_content, $wrapper ) !== false ) {
            $form_content = str_replace( 'id="'. $wrapper .'"', 'style="width: 100%; max-width: 1170px;"', $form_content );

            $pattern = '/<h2\s+class="wppb-form-title">.*?<\/h2>/i';
            $form_content = preg_replace($pattern, '', $form_content);
            break;
        }
    }

    return $form_content;
}
add_filter( 'wppb_register_activate_user_error_message2', 'pms_clear_ec_payment_form_pb_styling', 150 );


// Output heading for Subscription Plans field
add_action( 'pms_register_form_subscription_plans_field_before', 'pms_fd_output_subscription_plans_field_heading' );
function pms_fd_output_subscription_plans_field_heading( $atts ){

    $active_design = pms_get_active_form_design();

    if ( $active_design == 'form-style-default' )
        return;

    echo '<h3 class="pms-subscriptions-list-title">'. esc_html__( 'Select Your Subscription Plan', 'paid-member-subscriptions' ) .'</h3>';

}